/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/
// AviWriter.cpp : AVI writer sample application using the VideoStorage library.

#include <stdio.h>
#include <unistd.h>
#include "multicam.h"
#include "videostorage.h"
#include <stdlib.h>

class MultiCamAviWriter
{
    int images;
    Euresys::VideoStorage::VideoWriter *writer;
public:
    MultiCamAviWriter() : images(0), writer(0) {}
    ~MultiCamAviWriter() { delete writer; }

    void Init(const char *filename, MCHANDLE channel)
    {
        // Create a VideoWriter object to write the AVI 
        // with images produced by the channel
        writer = Euresys::VideoStorage::CreateAviWriter(filename, channel);
    }

    void ProcessingCallback(PMCSIGNALINFO SigInfo)
    {
        MCHANDLE surface;

        surface = SigInfo->SignalInfo;

        // write the image in the AVI
        try
        {
            writer->AddImage(surface);
        }
        catch(...)
        {
            fprintf(stderr, "Caught an exception in Processing callback");
        }
        printf("Acquired image #%04d\r", images++);
        fflush(stdout);
    }
};

// Show the scope of the sample program
void PrintIntroduction()
{
    printf( 
        "This program demonstrates the VideoStorage library.\n"
        "\n"
        "Run the program on the command line to acquire compressed "
        "images from a PICOLO Diligent and store them in "
        "capture.avi.\n"
        "\n"
        "Example requirements:\n"
        "- a PAL camera connected on VID1\n"
        "  (Change the CamFile parameter in the code if you are using a NTSC camera).\n"
        "- a PICOLO Diligent as the first board."
        "\n\n"
        );
}


// The rest relates to configuring MultiCam and starting the acquisitions.

void MCAPI ChannelCallback(PMCSIGNALINFO SigInfo)
{
    switch(SigInfo->Signal)
    {
    case MC_SIG_SURFACE_PROCESSING:
        static_cast<MultiCamAviWriter*>(SigInfo->Context)->ProcessingCallback(SigInfo);
        break;
    case MC_SIG_ACQUISITION_FAILURE:
        printf("Acquisition Failure !");
        break;
    default:
        break;
    }
}

void MultiCamInit() 
{
    // Initialize MultiCam driver
    McOpenDriver(NULL);

    McSetParamStr(MC_CONFIGURATION, MC_ErrorLog, "error.log");
}

void SetParam(MCHANDLE object, MCPARAMID id, int value)
{
    MCSTATUS status;
    status = McSetParamInt(object, id, value);
    if (status!=MC_OK)
    {
        fprintf(stderr, "Error 0x%08x on id 0x%08x\n", status, id);
        exit(1);
    }
}

MCHANDLE InitializeChannel()
{
    MCHANDLE channel;

    // Create a channel and associate it with the first board
    McCreate(MC_CHANNEL, &channel);
    SetParam(channel, MC_DriverIndex, 0);
    SetParam(channel, MC_Connector, MC_Connector_VID1);
    McSetParamStr(channel, MC_CamFile, "PAL");
    SetParam(channel, MC_ColorFormat, MC_ColorFormat_JPEG);

    SetParam(channel, MC_TrigMode, MC_TrigMode_IMMEDIATE);
    SetParam(channel, MC_NextTrigMode, MC_NextTrigMode_REPEAT);
    SetParam(channel, MC_SeqLength_Fr, MC_INDETERMINATE);

    // The memory allocation for the images is done automatically by Multicam 
    // when activating the channel.

    // Enable MultiCam signals
    SetParam(channel, MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, MC_SignalEnable_ON);
    SetParam(channel, MC_SignalEnable + MC_SIG_ACQUISITION_FAILURE, MC_SignalEnable_ON);

    return channel;
}

void RecordMovie()
{
    MultiCamAviWriter capture;
    MCHANDLE channel;

    channel = InitializeChannel();
    McRegisterCallback(channel, ChannelCallback, &capture);

    capture.Init("capture.avi", channel);

    // Record during 5 seconds
    printf("Starting the acquisitions:\n");
    SetParam(channel, MC_ChannelState, MC_ChannelState_ACTIVE);
    sleep(5);
    SetParam(channel, MC_ChannelState, MC_ChannelState_IDLE);
    sleep(1);
    printf("\nMovie recorded in capture.avi.\n");

    McDelete(channel);
}

int main(int argc, char* argv[])
{
    try
    {
        PrintIntroduction();

        MultiCamInit();
        RecordMovie();
        McCloseDriver();
    }
    catch(Euresys::VideoStorage::Exception &e)
    {
        fprintf(stderr, "Caught an exception:\n");
        fprintf(stderr, e.What());
        fprintf(stderr, "\n");
    }
    catch(...)
    {
        fprintf(stderr, "Caught unknown exception\n");
    }

    return 0;
}
