/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/


/***********************************************************************
  MultiCam Sample program.
  
  Acquires an image from the first board and save it in a ppm file.
  The board must come from Picolo family.
  
  The ppm file can be easily viewed with most image display programs,
  like gimp, kview, xv.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "/usr/local/euresys/multicam/include/multicam.h"

#ifndef BYTE
  #define BYTE unsigned char
#endif


/** The number of surfaces we will use.

Should be at least 3 for live acquisition.
**/
#define NSURFACES  3

int sizex, sizey;
int saved_images=0;


void AcquireImages();
void PostProcessImage(BYTE *pImage, int sizeX, int sizeY);
void ProcessImage(BYTE *pImage, int sizeX, int sizeY);
void MCAPI McCallback(PMCCALLBACKINFO CallBackInfo);
void SaveImage(BYTE *pImage, int sizeX, int sizeY);
MCSTATUS CreateSurfaces(MCHANDLE hChannel, MCHANDLE hSurfaces[], BYTE *pImages[]);


int main(int argc, char* argv[])
{
    MCSTATUS status;

    /* Initialize the MultiCam driver. 
       Do it once at the beginning of your program.
    */
    status = McOpenDriver(NULL);
    if(status != MC_OK)
    {
        fprintf(stderr,"Error couldn't initialize MultiCam\n");
        return -1;
    }

    /* Acquire the images and process them.*/
    AcquireImages();

    /* Close the MultiCam driver. */
    McCloseDriver();

    return 0;
}


/*
    Acquire images and process them.
    
    1. Create a channel and set its parameters
    2. Add Surfaces
    3. Start the image acquisition, images are processed int the callback
*/
void AcquireImages() 
{
    MCSTATUS status = 0;
    MCHANDLE hChannel = 0;
    MCHANDLE hSurfaces[NSURFACES];
    BYTE *pImages[NSURFACES];
    int i;

    sizex=0;
    sizey=0;

    /* Create a channel.
       Before that you might want to set the topology.
    */
    status = McCreate(MC_CHANNEL, &hChannel);
    if (status != MC_OK) goto Finalize;

    /* Link the channel to a board. Here we take the first board.*/
    status = McSetParamInt(hChannel, MC_DriverIndex, 0);
    if (status != MC_OK) goto Finalize;

    /* Select the video connector. Here the first video connector. */
    status = McSetParamInt(hChannel, MC_Connector, MC_Connector_VID1);

    /* Select the video standard. PAL or NTSC */
    status = McSetParamInt(hChannel, MC_Camera, MC_Camera_CAMERA_NTSC);
    if (status != MC_OK) goto Finalize;

    /* Set the color format. Here the program is written for RGB24. */
    status = McSetParamInt(hChannel, MC_ColorFormat, MC_ColorFormat_RGB24);
    if (status != MC_OK)
    {
        fprintf(stderr,"Wrong color format\n");
        goto Finalize;
    }

    /* Acquire full frames. Synchronising on the first available field. */
    status = McSetParamInt(hChannel, MC_GrabField, MC_GrabField_FRAME);
    if (status != MC_OK) goto Finalize;

    /* Here is the ideal place to adjust some other acquisition parameters,
       like image size, grab window, gain or offset.*/

    /* Get the image width and height. 
       Will be used to allocate the right amount of memory 
       and for processing the image
    */
    status= McGetParamInt(hChannel, MC_ImageSizeX, &sizex);
    if (status != MC_OK) goto Finalize;
    status= McGetParamInt(hChannel, MC_ImageSizeY, &sizey);
    if (status != MC_OK) goto Finalize;  

    CreateSurfaces(hChannel,hSurfaces,pImages);

    /* The number of images to acquire.
       MC_INDETERMINATE: indefinitely repeated frame acquisition
    */
    status = McSetParamInt (hChannel, MC_SeqLength_Fr, 1);
    if (status != MC_OK) goto Finalize;

    /* Register our Callback function for the MultiCam asynchronous signals.*/
    status = McRegisterCallback(hChannel, McCallback, NULL);
    if (status != MC_OK) goto Finalize;

    /* Enable the signals we need: 
       MC_SIG_SURFACE_PROCESSING: acquisition done and locked for processing
       MC_SIG_ACQUISITION_FAILURE: acquisition failed.
    */
    status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, 
        MC_SignalEnable_ON);
    if (status != MC_OK) goto Finalize;
    status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_ACQUISITION_FAILURE, 
        MC_SignalEnable_ON);
    if (status != MC_OK) goto Finalize;


    /*
        Start Acquisitions for this channel. 
        Acquisition will start when the channel is active.
    */
    fprintf(stderr,"McSetParamInt MC_ChannelState <MC_ChannelState_ACTIVE>\n");
    status = McSetParamInt(hChannel, MC_ChannelState, MC_ChannelState_ACTIVE);
    if (status != MC_OK) goto Finalize;

    fprintf(stderr,"WAITING for image processing callbacks\n");

    /* Wait 2 seconds. That should be enough for the acquisition to finish.
       In industrial programs, this will probably be replaced with some wait
       for the end of the application.
    */
    sleep(2);

    fprintf(stderr,"McSetParamInt MC_ChannelState <MC_ChannelState_IDLE>\n");
    status= McSetParamInt(hChannel, MC_ChannelState, MC_ChannelState_IDLE);

Finalize:
    if (status!=MC_OK)
    {
        printf("Error %d\n",status);
    }

    if (hChannel)
    {
        McDelete(hChannel);
    }

    for(i=0;i<NSURFACES;i++)
    {
        if(hSurfaces[i])
        {
            McDelete(hSurfaces[i]);
            free(pImages[i]);
        }
    }
}

/*
    Create surfaces with their memory buffers and add them to the
    cluster of the channel.
*/
MCSTATUS CreateSurfaces(MCHANDLE hChannel, MCHANDLE hSurfaces[NSURFACES], BYTE *pImages[NSURFACES])
{
    MCSTATUS status;
    int	surface, total_size = 0, buffer_size, buffer_pitch;

    /* Get the recommended sizes for the surfaces */
    status = McGetParamInt(hChannel, MC_BufferSize, &buffer_size);
    if (status != MC_OK) return status;

    status = McGetParamInt(hChannel, MC_BufferPitch, &buffer_pitch);
    if (status != MC_OK) return status;

    for (surface = 0; surface < NSURFACES; surface++)
    {
        /* Allocate memory */
        pImages[surface] = (BYTE *) malloc(buffer_size);
        if (pImages[surface] == NULL) return status;
        memset(pImages[surface], 0, buffer_size);

        /* Create a generic surface*/
        McCreate(MC_DEFAULT_SURFACE_HANDLE, &hSurfaces[surface]);

        /* Set a user context. 
           The programmer can get this value back for his own purposes. */
        McSetParamInt(hSurfaces[surface], MC_SurfaceContext, surface);

        /* Set the address of the memory allocated for this surface */
        status = McSetParamInt(hSurfaces[surface], MC_SurfaceAddr, (int)pImages[surface]);
        if (status != MC_OK) return status;

        /* Set the surface pitch and size */
        status = McSetParamInt(hSurfaces[surface], MC_SurfaceSize, buffer_size);
        if (status != MC_OK) return status;

        status = McSetParamInt(hSurfaces[surface], MC_SurfacePitch, buffer_pitch);
        if (status != MC_OK) return status;

        /* Add the surface to the cluster of the channel. */
        status = McSetParamInst(hChannel, MC_Cluster + surface, hSurfaces[surface]);
        if (status != MC_OK) return status;
    }
}

/*
    Callback function registered to our channel. 
    Called at the asynchronous signals.
*/
void MCAPI McCallback(PMCCALLBACKINFO CallBackInfo)
{
    MCHANDLE hSurface;
    BYTE *pImage;

    switch(CallBackInfo->Signal)
    {
    case MC_SIG_SURFACE_PROCESSING:
        fprintf(stderr,"Processing surface.\n");
        hSurface = (MCHANDLE) CallBackInfo->SignalInfo;
        McGetParamPtr(hSurface,MC_SurfaceAddr,(PVOID)&pImage);
        ProcessImage(pImage, sizex, sizey);
        break;
    case MC_SIG_ACQUISITION_FAILURE:
        fprintf(stderr, "Acquisition Failure. Is a video signal connected ?\n");
        break;
    default:
        fprintf(stderr,"Signal not handled:%d", CallBackInfo->Signal);
        break;
    }
}

void ProcessImage(BYTE *pImage, int sizeX, int sizeY)
{
    SaveImage(pImage, sizeX, sizeY);
}

/*
    Save the image in a .ppm file.
*/
void SaveImage(BYTE *pImage, int sizeX, int sizeY)
{
    FILE *f;
    char filename[256];
    int i;

    sprintf(filename,"img_%04d.ppm",saved_images++);
    f=fopen(filename,"w");

    fprintf(f,"P6\n%d\n%d\n255\n",sizeX,sizeY);
    for(i = 0; i < sizeX * sizeY * 3; i += 3)
    {
        fputc(pImage[i+2],f);
        fputc(pImage[i+1],f);
        fputc(pImage[i+0],f);
    }

    fclose(f);
}
